import ssl
import os
import random
import string
import yaml
from flask import Flask
from flask import jsonify
from flask import request
from flask import abort
import consul

from biomaj_user.user import BmajUser
from biomaj_core.utils import Utils

config_file = 'config.yml'
if 'BIOMAJ_CONFIG' in os.environ:
    config_file = os.environ['BIOMAJ_CONFIG']

config = None
with open(config_file, 'r') as ymlfile:
    config = yaml.load(ymlfile)
    Utils.service_config_override(config)

BmajUser.set_config(config)

app = Flask(__name__)


def consul_declare(config):
    if config['consul']['host']:
        consul_agent = consul.Consul(host=config['consul']['host'])
        consul_agent.agent.service.register(
            'biomaj-user',
            service_id=config['consul']['id'],
            address=config['web']['hostname'],
            port=config['web']['port'],
            tags=[
                'biomaj',
                'api',
                'traefik-int.backend=biomaj-user',
                'traefik-int.frontend.rule=PathPrefix:/api/user',
                'traefik-int.enable=true',
            ]
        )
        check = consul.Check.http(url='http://' + config['web']['hostname'] + ':' + str(config['web']['port']) + '/api/user', interval=20)
        consul_agent.agent.check.register(config['consul']['id'] + '_check', check=check, service_id=config['consul']['id'])


consul_declare(config)


@app.route('/api/user', methods=['GET'])
def ping():
    return jsonify({'msg': 'pong'})


@app.route('/api/user/info/user', methods=['GET'])
def list_users():
    '''
    List users
    '''
    users = BmajUser.list()
    for user in users:
        del user['_id']
        del user['hashed_password']
    return jsonify({'users': users})


@app.route('/api/user/info/user/<user>', methods=['GET'])
def get_user(user):
    '''
    Get user info
    '''
    user = BmajUser(user)
    if not user.user:
        abort(404)
    del user.user['_id']
    if 'hashed_password' in user.user:
        del user.user['hashed_password']
    return jsonify({'user': user.user})


@app.route('/api/user/info/user/<user>', methods=['POST'])
def create_user(user):
    '''
    Create a user
    '''
    user = BmajUser(user)
    param = request.get_json()
    if 'password' not in param:
        param['password'] = ''.join(random.SystemRandom().choice(string.ascii_uppercase + string.digits) for _ in range(10))
    if 'email' not in param:
        param['email'] = None
    if not user.user:
        user.create(password=param['password'], email=param['email'])
    del user.user['_id']
    if 'hashed_password' in user.user:
        del user.user['hashed_password']
    return jsonify({'user': user.user, 'password': param['password']})


@app.route('/api/user/bind/user/<user>', methods=['POST'])
def bind_user(user):
    '''
    Bind a user with his password or API Key. Post parameters dict:

    {'type': 'password|apikey', 'value': 'XXXX'}


    '''
    user = BmajUser(user)
    params = request.get_json()
    check = False
    if params['type'] == 'password':
        check = user.check_password(params['value'])
    else:
        check = user.check_apikey(params['value'])
    if not check:
        abort(401)
    del user.user['_id']
    if 'hashed_password' in user.user:
        del user.user['hashed_password']
    return jsonify({'user': user.user})


@app.route('/api/user/info/apikey/<apikey>', methods=['GET'])
def get_user_by_apikey(apikey):
    '''
    Get a user from his api key
    '''
    user = BmajUser.get_user_by_apikey(apikey)
    if user:
        del user['_id']
        if 'hashed_password' in user:
            del user['hashed_password']
    return jsonify({'user': user})


if __name__ == "__main__":
    context = None
    if config['tls']['cert']:
        context = ssl.SSLContext(ssl.PROTOCOL_TLSv1_2)
        context.load_cert_chain(config['tls']['cert'], config['tls']['key'])
    app.run(host='0.0.0.0', port=config['web']['port'], ssl_context=context, threaded=True, debug=config['web']['debug'])



import argparse
from argparse import Namespace as options
import os
import random
import yaml
import string
import sys
import bcrypt
from tabulate import tabulate
from biomaj_user.user import BmajUser
from biomaj_core.utils import Utils
SUPPORTED_ACTIONS = ['add', 'create', 'delete', 'remove', 'rm', 'renew', 'update', 'view']


def main():
    """This is the main function treating arguments passed on the command line."""
    description = "BioMAJ user: Manager users."
    parser = argparse.ArgumentParser(description=description)
    # Options without value
    parser.add_argument('-A', '--action', dest="action", default=None,
                        help="Action to perform for user " + str(SUPPORTED_ACTIONS) +
                             "'renew': Create new api key",
                        required=True)
    parser.add_argument('-C', '--config', dest="config", metavar='</path/to/config.yml>', type=str,
                        help="Path to config.yml. By default read from env variable BIOMAJ_CONFIG")
    parser.add_argument('-E', '--email', dest="email", type=str,
                        help="User email, optional")
    parser.add_argument('-U', '--user', dest="user", metavar='<username>', type=str,
                        required=True, help="User name to manage")
    parser.add_argument('-P', '--password', dest="passwd", metavar="<password>", type=str,
                        help="User password to use when creating new user. If not given, automatically generated, accepts env variable BIOMAJ_USER_PASSWORD env variable")
    parser.parse_args(namespace=options)
    if not len(sys.argv) > 1:
        parser.print_help()
        sys.exit(1)
    if options.action not in SUPPORTED_ACTIONS:
        print("Unsupported action '%s'" % str(options.action))
        sys.exit(1)

    if options.config:
        config = options.config
    elif 'BIOMAJ_CONFIG' in os.environ:
        config = os.environ['BIOMAJ_CONFIG']
    else:
        config = 'config.yml'
    with open(config, 'r') as ymlfile:
        config = yaml.load(ymlfile)
        Utils.service_config_override(config)

    BmajUser.set_config(config)
    user = BmajUser(options.user)
    if options.action in ['add', 'create']:
        if user.user is None:
            if options.passwd is None:
                if 'BIOMAJ_USER_PASSWORD' in os.environ:
                    options.passwd = os.environ['BIOMAJ_USER_PASSWORD']
                else:
                    options.passwd = ''.join(random.SystemRandom().choice(string.ascii_uppercase + string.digits)
                                         for _ in range(10))
            user.create(options.passwd, email=options.email)
            print("User successfully created")
            print(tabulate([["User", "Password", "API Key"],
                            [user.user['id'], str(options.passwd), str(user.user['apikey'])]],
                           headers="firstrow", tablefmt="psql"))
            sys.exit(0)
        else:
            print("User %s already exist" % user.user['id'])
            sys.exit(1)

    if user.user is None:
        print("[%s] User %s does not exist" % (str(options.action), str(options.user)))
        sys.exit(1)

    if options.action in ['delete', 'remove', 'rm']:
        user.remove()
        print("User %s successfully deleted" % user.user['id'])
    if options.action == 'update':
        update = {}
        if options.passwd:
            update['hashed_password'] = bcrypt.hashpw(options.passwd, user.user['hashed_password'])
        if options.email:
            update['email'] = options.email
        if update.items():
            BmajUser.users.update({'id': user.user['id']}, {'$set': update})
            print("User %s successfully updated" % str(user.user['id']))
        else:
            print("[%s] User %s not updated" % (str(options.action), str(options.user)))
    if options.action == 'renew':
        user.renew_apikey()
        user = BmajUser(user.user['id'])
        print("[%s] User %s, successfully renewed API key: '%s'" %
              (str(options.action), str(user.user['id']), str(user.user['apikey'])))
    if options.action == 'view':
        print(tabulate([["User", "Email", "API Key", "LDAP"],
                        [str(user.user['id']), str(user.user['email']),
                         str(user.user['apikey']), str(user.user['is_ldap'])]],
                       headers="firstrow", tablefmt="psql"))
    sys.exit(0)

if __name__ == '__main__':
    main()



from builtins import str
from builtins import object
import bcrypt
import logging
import random
import string

from pymongo import MongoClient


class BmajUser(object):
    """
    Biomaj User
    """

    config = None

    client = None
    db = None
    banks = None
    users = None

    @staticmethod
    def set_config(config):
        BmajUser.config = config
        BmajUser.client = MongoClient(BmajUser.config['mongo']['url'])
        BmajUser.db = BmajUser.client[BmajUser.config['mongo']['db']]
        BmajUser.banks = BmajUser.db.banks
        BmajUser.users = BmajUser.db.users

    @staticmethod
    def get_user_by_apikey(apikey):
        return BmajUser.users.find_one({'apikey': apikey})

    def __init__(self, user):
        self.id = user
        self.user = BmajUser.users.find_one({'id': user})

        ldap_server = None
        con = None
        if not self.user and BmajUser.config['ldap']['host']:
            # Check if in ldap
            from ldap3 import Server, Connection, SYNC, SUBTREE, ALL
            try:
                ldap_host = BmajUser.config['ldap']['host']
                ldap_port = BmajUser.config['ldap']['port']
                ldap_server = Server(ldap_host, port=ldap_port, get_info=ALL)
                con = Connection(ldap_server, auto_bind=True, client_strategy=SYNC, check_names=True)
            except Exception as err:
                logging.error(str(err))
                self.user = None
            ldap_dn = BmajUser.config['ldap']['dn']
            base_dn = 'ou=People,' + ldap_dn
            ldapfilter = "(&(|(uid=" + user + ")(mail=" + user + ")))"
            try:
                attrs = ['mail']
                con.search(base_dn, ldapfilter, SUBTREE, attributes=attrs)
                if con.response:
                    ldapMail = None
                    for r in con.response:
                        # user_dn = str(r['dn'])
                        if 'mail' not in r['attributes']:
                            logging.error('Mail not set for user ' + user)
                        else:
                            ldapMail = r['attributes']['mail'][0]
                    self.user = {
                        'id': user,
                        'email': ldapMail,
                        'is_ldap': True,
                        'apikey': ''.join(random.SystemRandom().choice(string.ascii_uppercase + string.digits) for _ in range(10))
                    }
                    self.user['_id'] = self.users.insert(self.user)

                else:
                    self.user = None
            except Exception as err:
                logging.error(str(err))
            if con:
                con.unbind()

    @staticmethod
    def user_remove(user_name):
        """
        Remove a user from db

        :param user_name: user name
        :type user_name: str
        """
        BmajUser.users.remove({'id': user_name})

    @staticmethod
    def list():
        """
        Get users
        """
        users = []
        user_list = BmajUser.users.find()
        for user in user_list:
            users.append(user)
        return users

    def check_apikey(self, apikey):
        if self.user is None:
            return False
        if 'apikey' not in self.user:
            return False
        if self.user['apikey'] == apikey:
            return True
        else:
            return False

    def check_password(self, password):
        if self.user is None:
            return False

        if self.user['is_ldap']:
            con = None
            ldap_server = None

            from ldap3 import Server, Connection, SIMPLE, SYNC, SUBTREE, ALL
            from ldap3.core.exceptions import LDAPBindError
            try:
                ldap_host = BmajUser.config['ldap']['host']
                ldap_port = BmajUser.config['ldap']['port']
                ldap_server = Server(ldap_host, port=ldap_port, get_info=ALL)
                con = Connection(ldap_server, auto_bind=True, client_strategy=SYNC, check_names=True)
            except Exception as err:
                logging.error(str(err))
                return False
            ldap_dn = BmajUser.config['ldap']['dn']
            base_dn = 'ou=People,' + ldap_dn
            ldapfilter = "(&(|(uid=" + self.user['id'] + ")(mail=" + self.user['id'] + ")))"

            try:
                attrs = ['mail']
                con.search(base_dn, ldapfilter, SUBTREE, attributes=attrs)
                user_dn = None
                # ldapMail = None
                # ldapHomeDirectory = None
                for r in con.response:
                    user_dn = str(r['dn'])
                    # ldapMail = r['attributes']['mail'][0]

                con.unbind()
                con = Connection(ldap_server, auto_bind=True, read_only=True, client_strategy=SYNC, user=user_dn, password=password, authentication=SIMPLE, check_names=True)
                con.unbind()

                if user_dn:
                    return True
            except LDAPBindError as err:
                logging.error('Bind error: ' + str(err))
                return False
            except Exception as err:
                logging.error('Bind error: ' + str(err))
                return False

        else:
            hashed = bcrypt.hashpw(password, self.user['hashed_password'])
            if hashed == self.user['hashed_password']:
                return True
            else:
                return False

    def remove(self):
        if self.user is None:
            return False
        BmajUser.users.remove({'_id': self.user['_id']})
        return True

    def create(self, password, email=''):
        """
        Create a new user
        """
        hashed = bcrypt.hashpw(password, bcrypt.gensalt())
        if self.user is None:
            self.user = {
                'id': self.id,
                'hashed_password': hashed,
                'email': email,
                'is_ldap': False,
                'apikey': ''.join(random.SystemRandom().choice(string.ascii_uppercase + string.digits) for _ in range(10))
            }
            self.user['_id'] = BmajUser.users.insert(self.user)

    def renew_apikey(self):
        api_key = ''.join(random.SystemRandom().choice(string.ascii_uppercase + string.digits) for _ in range(10))
        BmajUser.users.update({'_id': self.user['_id']}, {'$set': {'apikey': api_key}})
